import sys
import os
import bisect

import numpy as np
import pandas as pd

from PyQt5 import QtGui#,# uic
from PyQt5.QtCore import QThread, pyqtSignal, Qt, QRect, QTimer
import pyqtgraph as pg
import inspect
import h5py

# todo test if these work without being called from main_gui at pyecog level
if __name__ != '__main__':
    from . import check_preds_design, loading_subwindow, convert_ndf_window
    from . import subwindows
    from .context import H5File

else:
    import check_preds_design, loading_subwindow, convert_ndf_window
    import subwindows
    from context import H5File

#from ndf.datahandler import DataHandler
#from pyecog.visualisation.pyqtgraph_playing import HDF5Plot

#TODO - you are currently loading the entire h5 file into memory..
class MainGui(QtGui.QMainWindow, check_preds_design.Ui_MainWindow):
    def __init__(self, parent=None):
        pg.setConfigOption('background', 'w')
        pg.setConfigOption('foreground', 'k')
        super(MainGui, self).__init__(parent)
        self.setupUi(self)
        self.scroll_flag = -1
        self.top_splitter.setSizes([200,600])
        self.bottom_splitter.setSizes([500,300])
        self.full_splitter.setSizes([300,200,200])
        if self.blink_box.isChecked():
            self.blink      = 1
        else:
            self.blink      = -1
        self.scroll_sign = 1
        self.timer = QTimer()
        self.timer.timeout.connect(self.simple_scroll)
        self.blink_box.stateChanged.connect(self.blink_box_change)
        self.scroll_speed_box.valueChanged.connect(self.scroll_speed_change)
        self.xrange_spinBox.valueChanged.connect(self.xrange_change)
        self.tid_spinBox.valueChanged.connect(self.tid_spinBox_handling)

        self.fs = None # change !
        self.data_obj = None
        self.predictions_df = None
        self.h5directory = None
        self.tree_items = []

        if os.path.exists('/Volumes/G-DRIVE with Thunderbolt/2017 pyecog demo/'):
            self.home = '/Volumes/G-DRIVE with Thunderbolt/2017 pyecog demo/'
        else:
            self.home = os.getcwd()


        # Hook up the file bar stuff here
        self.actionLoad_Predictions.triggered.connect(self.load_predictions_gui)
        self.actionSave_annotations.triggered.connect(self.master_tree_export_csv)
        self.actionLoad_Library.triggered.connect(self.load_seizure_library)
        self.actionLoad_h5_folder.triggered.connect(self.load_h5_folder) # this is still to do in its entireity
        self.actionSet_default_folder.triggered.connect(self.set_home)

        # Hook up analyse menu bar to functions here
        self.actionConvert_ndf_to_h5.triggered.connect(self.convert_ndf_folder_to_h5)
        self.actionLibrary_logistics.triggered.connect(self.load_library_management_subwindow)
        self.actionClassifier_components.triggered.connect(self.load_clf_subwindow)
        self.actionAdd_features_to_h5_folder.triggered.connect(self.load_add_prediction_features_subwindow)

        self.plot_1 = self.GraphicsLayoutWidget.addPlot()
        self.plot_overview = self.overview_plot.addPlot()
        #self.tid_box.setValue(6)
        #self.traceSelector.valueChanged.connect(self.plot_traces)
        #self.channel_selector.valueChanged.connect(self.plot_traces)
        self.treeWidget.itemSelectionChanged.connect(self.master_tree_selection)

        self.predictions_up = False
        self.library_up = False
        self.file_dir_up = False

        #self.debug_load_pred_files()

        # Below resizes to better geometries - should really use this to save etc!
        # doesnt quite work correctly!

        self.print_widget_coords = False # use this to print out coords when clicking the plot stuff

    def not_done_yet(self):
        QtGui.QMessageBox.information(self,"Not implemented, lazy!", "Not implemented yet! Jonny has been lazy!")

    def load_clf_subwindow(self):
        child = subwindows.ClfWindow()
        child.show()
        child.home = self.home
        if child.exec_():
            print('exec_() was called')
        return 0

    def load_add_prediction_features_subwindow(self):
        child = subwindows.AddPredictionFeaturesWindow()
        child.show()
        child.home = self.home
        if child.exec_():
            print('exec_() was called')
        return 0

    def load_library_management_subwindow(self):
        child = subwindows.LibraryWindow()
        child.show()
        child.home = self.home
        if child.exec_():
            print('exec_was called')
        return 0

    def convert_ndf_folder_to_h5(self):
        child = subwindows.ConvertingNDFsWindow()
        child.show()
        child.home = self.home
        if child.exec_():
            print('exec_was called')
        return 0

    def set_home(self):
        self.home = QtGui.QFileDialog.getExistingDirectory(self, "Set a default folder to load from", self.home)


    def load_h5_folder(self):
        self.h5directory_to_load = QtGui.QFileDialog.getExistingDirectory(self, "Pick a h5 folder", self.home)
        if self.h5directory_to_load == '':
            print('No folder selected')
            return 0

        self.clear_QTreeWidget()
        fnames = [f for f in os.listdir(self.h5directory_to_load) if f.endswith('.h5') if not f.startswith('.') ]
        print(fnames)
        for i,fname in enumerate(fnames):
            #fpath = os.path.join(self.h5directory_to_load,fname)
            try:
                tids = eval('['+fname.split(']')[0].split('[')[1]+']')
                self.populate_tree_items_list_from_h5_folder(i,fname, tids)    # this just populates self.tree_items
            except:
                print('Failed to add: '+ str(fname))

        self.treeWidget.addTopLevelItems(self.tree_items)

        self.predictions_up = False
        self.library_up = False
        self.file_dir_up = True

    def populate_tree_items_list_from_h5_folder(self,index,fpath,tids):
        self.treeWidget.setColumnCount(3)
        self.treeWidget.setHeaderLabels(['index', 'tids', 'fname'])

        fname_entry = [str(fpath)] # us this pointless!?
        details_entry = [str(index),
                         str(tids), # bad, should make only tid having one explicit
                         str(fpath)]

        item = QtGui.QTreeWidgetItem(details_entry)
        item.setFirstColumnSpanned(True)

        self.tree_items.append(item)

    def load_seizure_library(self):

        self.library = QtGui.QFileDialog.getOpenFileName(self, "Pick a h5 library file", self.home)[0]
        if self.library is '':
            print('nothing selected')
            return 0
        print(self.library)
        print(type(self.library))
        self.clear_QTreeWidget()
        # todo here check if got chunked thing from lirbary.. if not, throw messagebox error and return 0
        with h5py.File(self.library) as f:
            group_names = list(f.keys())
            groups = [f[key] for key in group_names]

            for i, group in enumerate(groups):
                for seizure_i in range(group.attrs['precise_annotation'].shape[0]):
                    row = {'name' : group_names[i],
                           'start': group.attrs['precise_annotation'][seizure_i, 0],
                           'end'  : group.attrs['precise_annotation'][seizure_i, 1],
                           'tid'  : group.attrs['tid'],
                           'index': i, # not sure if this is gonna work/ not consistent with the predictions
                           'chunk_start': group.attrs['chunked_annotation'][seizure_i, 0],
                           'chunk_end': group.attrs['chunked_annotation'][seizure_i, 1],
                           }
                    self.populate_tree_items_from_library(row)
            self.treeWidget.addTopLevelItems(self.tree_items)

        self.predictions_up = False
        self.library_up = True
        self.file_dir_up = False

    def populate_tree_items_from_library(self, row):

        self.treeWidget.setColumnCount(7)
        self.treeWidget.setHeaderLabels(['index','start','end','duration','chunk_start','chunk_end', 'tid','name'])
        details_entry = [str(row['index']),
                         str(row['start']),
                         str(row['end']),
                         str((row['end'] - row['start']) ),
                         str(row['chunk_start']),
                         str(row['chunk_end']),
                         str(row['tid']),
                         str(row['name'])]

        item = QtGui.QTreeWidgetItem(details_entry)
        item.setFirstColumnSpanned(True)

        self.tree_items.append(item)

    def master_tree_export_csv(self):
        if self.predictions_up:
            self.predictions_tree_export_csv()
        elif self.library_up:
            self.library_tree_export_csv()
        elif self.file_dir_up:
            self.file_tree_export_csv()

    def file_tree_export_csv(self):
        pass

    def library_tree_export_csv(self):
        if self.h5directory:
            default_dir = os.path.dirname(self.h5directory)
        else:
            default_dir = ""
        save_name = QtGui.QFileDialog.getSaveFileName(self,'Save library details in a .csv file',default_dir)[0]
        if save_name is '':
            print('nothing selected')
            return 0
        # now build dataframe from the tree
        root = self.treeWidget.invisibleRootItem()
        child_count = root.childCount()
        index, start, end, tid, fname, duration = [],[],[],[],[], []
        for i in range(child_count):
            item = root.child(i)
            index.append(item.text(0))
            start.append(item.text(1))
            end.append(item.text(2))
            tid.append(item.text(6))
            fname.append(item.text(7))
            duration.append(item.text(3))
        exported_df = pd.DataFrame(data = np.vstack([fname,start,end,duration,tid]).T,columns = ['filename','start','end','duration','transmitter'] )

        save_name = save_name.strip('.csv')
        exported_df.to_csv(save_name+'.csv')

    def predictions_tree_export_csv(self):
        if self.h5directory:
            default_dir = os.path.dirname(self.h5directory)
        else:
            default_dir = ""
        save_name = QtGui.QFileDialog.getSaveFileName(self,'Save annotation .csv file',default_dir)[0]
        if save_name is '':
            print('nothing selected')
            return 0
        # now build dataframe from the tree
        root = self.treeWidget.invisibleRootItem()
        child_count = root.childCount()
        index, start, end, tid, fname, duration = [],[],[],[],[], []
        for i in range(child_count):
            item = root.child(i)
            index.append(item.text(0))
            start.append(item.text(1))
            end.append(item.text(2))
            tid.append(item.text(4))
            fname.append(item.text(5))
            duration.append(item.text(3))
        exported_df = pd.DataFrame(data = np.vstack([index,fname,start,end,duration,tid]).T,columns = ['old_index','filename','start','end','duration','transmitter'] )

        save_name = save_name.strip('.csv')
        exported_df.to_csv(save_name+'.csv')

    def master_tree_selection(self):
        if not self.deleteing:                     # this is a hack as was being called as I was clearing the items
            if self.predictions_up:
                self.tree_selection_predictions()
            elif self.library_up:
                self.tree_selection_library()
            elif self.file_dir_up:
                self.tree_selection_file_dir()

    def get_next_tree_item(self):
        print('try to grab next treewidget item!')

    def tree_selection_file_dir(self):
        # this method does too much
        "grab tree detail and use to plot"

        tids = self.treeWidget.currentItem().text(1)
        self.valid_h5_tids = eval(tids)
        self.handle_tid_for_file_dir_plotting() # this will automatically call the plotting by changing the v


    def load_filedir_h5_file(self,tid):
        '''
        Splitting tree_selection_file_dir up so changing the tid spinBox can reload easily
        '''
        import time
        start = time.time()
        fields = self.treeWidget.currentItem()
        path = fields.text(2)
        index = float(fields.text(0))
        fpath = os.path.join(self.h5directory_to_load, path)
        h5 = H5File(fpath)
        data_dict = h5[tid]
        self.fs = eval(h5.attributes['fs_dict'])[tid]
        self.add_data_to_plots(data_dict['data'], data_dict['time'])
        xrange = self.xrange_spinBox.value()
        self.plot_1.setXRange(0, xrange)
        self.plot_1.setTitle(str(index)+' - '+ fpath+ '\n')
        self.plot_overview.setTitle('Overview of file: '+str(index)+' - '+ fpath)
        self.updatePlot()


    def handle_tid_for_file_dir_plotting(self):
        # this is called when clicking on the tree structure
        current_spinbox_id = self.tid_spinBox.value()
        if current_spinbox_id not in self.valid_h5_tids:
            self.tid_spinBox.setValue(self.valid_h5_tids[0])
            print('warning - file id changed')
        else:
            self.tid_spinBox_handling()

    def tid_spinBox_handling(self):
        # tid_spinbox.valueChanged connects to here
        new_val = self.tid_spinBox.value()
        if new_val < min(self.valid_h5_tids):
            new_tid = self.valid_h5_tids[-1]
        else:
            i = bisect.bisect_left(self.valid_h5_tids,new_val)
            new_tid = self.valid_h5_tids[i%len(self.valid_h5_tids)]
        self.tid_spinBox.setValue(new_tid)
        self.load_filedir_h5_file(new_tid)

    def tree_selection_library(self):
        seizure_buffer = 5 # seconds either side of seizure to plot
        current_item = self.treeWidget.currentItem()
        fields = current_item

        index = int(float(fields.text(0)))
        start = float(fields.text(1))
        end = float(fields.text(2))
        duration = float(fields.text(3))
        chunk_start = float(fields.text(4))
        chunk_end = float(fields.text(5))
        tid = float(fields.text(6))
        key = fields.text(7)

        with h5py.File(self.library) as f:
            dataset = f[key]
            self.fs = dataset.attrs['fs']
            # todo, then assumes you have calculated labels need to be calculated second?
            # i guess you can just add labels before?

            self.plot_1.clear()
            self.bx1 = self.plot_1.getViewBox()

            data = dataset['data'][:]
            time = np.linspace(0, data.shape[0]/self.fs, data.shape[0])
            self.add_data_to_plots(data,time)

            start_pen = pg.mkPen((85, 168, 104), width=3, style= Qt.DashLine)
            end_pen = pg.mkPen((210,88,88), width=3, style= Qt.DashLine)
            coarse_pen = pg.mkPen((210,210,210), width=3, style= Qt.DashLine)

            self.start_line = pg.InfiniteLine(pos=start, pen =start_pen, movable=True,label='{value:0.2f}',
                           labelOpts={'position':0.1, 'color': (200,200,100), 'fill': (200,200,200,0), 'movable': True})
            self.end_line = pg.InfiniteLine(pos=end, pen = end_pen, movable=True,label='{value:0.2f}',
                           labelOpts={'position':0.1, 'color': (200,200,100), 'fill': (200,200,200,0), 'movable': True})

            self.start_coarse = pg.InfiniteLine(pos=chunk_start, pen =coarse_pen, movable=False,label='{value:0.2f}',
                           labelOpts={'position':0.1, 'color': (200,200,100), 'fill': (200,200,200,0), 'movable': True})
            self.end_coarse   = pg.InfiniteLine(pos=chunk_end, pen =coarse_pen, movable=False,label='{value:0.2f}',
                           labelOpts={'position':0.1, 'color': (200,200,100), 'fill': (200,200,200,0), 'movable': True})

            self.plot_1.addItem(self.start_line)
            self.plot_1.addItem(self.end_line)
            self.plot_1.addItem(self.start_coarse)
            self.plot_1.addItem(self.end_coarse)
            self.start_line.sigPositionChanged.connect(self.update_tree_element_start_time)
            self.end_line.sigPositionChanged.connect(self.update_tree_element_end_time)
            self.plot_1.setXRange(chunk_start-seizure_buffer, chunk_end+seizure_buffer)
            self.plot_1.setTitle(str(index)+' - '+ key+ '\n' + str(start)+' - ' +str(end))
            self.plot_overview.setTitle('Overview of file: '+str(index)+' - '+ key)
            self.updatePlot()

    def tree_selection_predictions(self):
        # this method does too much
        "grab tree detail and use to plot"
        seizure_buffer = 5 # seconds either side of seizure to plot
        current_item = self.treeWidget.currentItem()

        fields = current_item
        tid = int(float(fields.text(4)))
        start = float(fields.text(1))
        end = float(fields.text(2))
        index = float(fields.text(0))
        # duration is fields.text(3)
        fpath = os.path.join(self.h5directory, fields.text(5))

        h5 = H5File(fpath)
        data_dict = h5[tid]
        self.fs = eval(h5.attributes['fs_dict'])[tid]

        self.add_data_to_plots(data_dict['data'], data_dict['time'])
        start_pen = pg.mkPen((85, 168, 104), width=3, style= Qt.DashLine)
        end_pen = pg.mkPen((210,88,88), width=3, style= Qt.DashLine)
        self.start_line = pg.InfiniteLine(pos=start, pen =start_pen, movable=True,label='{value:0.2f}',
                       labelOpts={'position':0.1, 'color': (200,200,100), 'fill': (200,200,200,0), 'movable': True})
        self.end_line = pg.InfiniteLine(pos=end, pen = end_pen, movable=True,label='{value:0.2f}',
                       labelOpts={'position':0.1, 'color': (200,200,100), 'fill': (200,200,200,0), 'movable': True})
        # todo add all lines per file in one go - more than one seizure
        self.plot_1.addItem(self.start_line)
        self.plot_1.addItem(self.end_line)
        self.start_line.sigPositionChanged.connect(self.update_tree_element_start_time)
        self.end_line.sigPositionChanged.connect(self.update_tree_element_end_time)

        self.plot_1.setXRange(start-seizure_buffer, end+seizure_buffer)
        self.plot_1.setTitle(str(index)+' - '+ fpath+ '\n' + str(start)+' - ' +str(end))
        self.plot_overview.setTitle('Overview of file: '+str(index)+' - '+ fpath)
        self.updatePlot()

    def add_data_to_plots(self, data, time):
        self.plot_1.clear()
        self.bx1 = self.plot_1.getViewBox()
        hdf5_plot = HDF5Plot(parent = self.plot_1, viewbox = self.bx1)
        hdf5_plot.setHDF5(data, time, self.fs)
        self.plot_1.addItem(hdf5_plot)
        self.plot_1.setLabel('left', 'Voltage (uV)')
        self.plot_1.setLabel('bottom','Time (s)')

        # hit up the linked view here
        self.plot_overview.clear()
        self.plot_overview.enableAutoRange(False,False)
        self.plot_overview.setXRange(0,3600) # hardcoding in the hour here...
        self.plot_overview.setMouseEnabled(x = False, y= True)
        self.bx_overview = self.plot_overview.getViewBox()
        hdf5_plotoverview = HDF5Plot(parent = self.plot_overview, viewbox = self.bx_overview)
        hdf5_plotoverview.setHDF5(data, time, self.fs)
        self.plot_overview.addItem(hdf5_plotoverview)
        self.plot_overview.setLabel('left', 'Voltage (uV)')
        self.plot_overview.setLabel('bottom','Time (s)')
        # mousePressEvent,mouseDoubleClickEvent ,sigMouseClicked,sigMouseMoved,wheelEvent
        self.proxy = pg.SignalProxy(self.plot_overview.scene().sigMouseClicked,rateLimit=30,slot=self.mouse_click_in_overview)
        #print(dir(self.plot_overview.scene()))

        self.lr = pg.LinearRegionItem(self.plot_1.getViewBox().viewRange()[0])
        self.lr.setZValue(-10)
        self.plot_overview.addItem(self.lr)
        # is this good practice?
        self.lr.sigRegionChanged.connect(self.updatePlot)
        self.plot_1.sigXRangeChanged.connect(self.updateRegion)
        self.updatePlot()


    # these two methods are for the lr plot connection, refactor names
    def updatePlot(self):
        self.plot_1.setXRange(*self.lr.getRegion(), padding=0)
    def updateRegion(self):
        self.lr.setRegion(self.plot_1.getViewBox().viewRange()[0])

    def update_tree_element_start_time(self):
        tree_row = self.treeWidget.currentItem()
        tree_row.setText(1,'{:.2f}'.format(self.start_line.x()))
        self.update_tree_element_duration()

    def update_tree_element_end_time(self):
        tree_row = self.treeWidget.currentItem()
        tree_row.setText(2,'{:.2f}'.format(self.end_line.x()))
        self.update_tree_element_duration()

    def update_tree_element_duration(self):
        tree_row = self.treeWidget.currentItem()
        duration = float(tree_row.text(2))-float(tree_row.text(1))
        tree_row.setText(3, '{:.2f}'.format(duration))

    def plot_traces(self, data_dict):

        if not self.holdPlot.isChecked():
            self.plot_1.clear()
        # here you need to add the h5 file class with downsampling

        curve1 = HDF5Plot()#parent = self.plot_1, viewbox = bx1)
        curve1.setHDF5(data_dict['data'], data_dict['time'], self.fs)
        self.plot_1.addItem(hdf5_plot)

        #self.plot_1.addItem(pg.PlotCurveItem(data_dict['time'], data_dict['data']))
        self.plot_1.setXRange(row['Start'], row['End'])
        #self.plot_1.ti

    def clear_QTreeWidget(self):
        # not sure if i need this top bit
        self.deleteing = True
        if self.treeWidget.currentItem():
            current_item = self.treeWidget.currentItem()
            root = self.treeWidget.invisibleRootItem()
            root.removeChild(current_item)

        root = self.treeWidget.invisibleRootItem()
        n_kids = root.childCount()
        for i in np.arange(n_kids)[::-1]:
            child = self.treeWidget.topLevelItem(i)
            root.removeChild(child)

        self.tree_items = []
        self.deleteing = False

    def load_predictions_gui(self):
        ''' here make the window for choosing h5 file directory and predictions '''
        # we want something displaying the two files and lets you optionally change the h5 folder.
        print('loading new window!')
        child = subwindows.LoadingSubwindow()
        child.show()
        child.home = self.home # will this inherit? :p
        if child.exec_():
            print(child.predictions_fname)
            self.h5directory = child.h5directory
            self.predictions_fname = child.predictions_fname

            self.update_h5_folder_display()
            self.update_predictionfile_display()
            self.load_pred_file()

    def update_h5_folder_display(self):
        self.h5_folder_display.setText(str(self.h5directory))

    def update_predictionfile_display(self):
        self.predictions_file_display.setText(str(self.predictions_fname))

    def load_pred_file(self):
        self.clear_QTreeWidget()
        if self.predictions_fname.endswith('.csv'):
            self.predictions_df = pd.read_csv(self.predictions_fname)
        elif self.predictions_fname.endswith('.xlsx'):
            self.predictions_df = pd.read_excel(self.predictions_fname)
        else:
            print('Please select .csv or .xlsx file')
            return 0
        self.update_predictionfile_display()
        self.predictions_df['Index'] = self.predictions_df.index
        if self.h5directory is None:
            self.set_h5_folder()

        for i,row in list(self.predictions_df.iterrows()):
            fpath = os.path.join(self.h5directory,row['Filename'])
            #TODO : decide what to do with tids, this is not thought out at the moment
            #So not bothering to load the tids here as should be one only per seizure... can either  load on demand
            # or use only for the data explorer stuff. Can maybe have dynamic when you click to see the full tree.
            # problem is with files with many false positives, spend time loading for too long!
            # or have a button for this...

            #h5 = H5File(fpath)
            #tids = h5.attributes['t_ids']
            tids = [int(fpath.split(']')[0].split('[')[1])]
            s,e = row['Start'], row['End']
            self.populate_tree_items_list_from_predictions(row, tids)    # this just populates self.tree_items
        self.treeWidget.addTopLevelItems(self.tree_items)

        self.predictions_up = True
        self.library_up = False
        self.file_dir_up = False

    def debug_load_pred_files(self): # stripped down version of the above for debugging gui code
        self.clear_QTreeWidget()
        try:
            self.h5directory = '/Volumes/G-DRIVE with Thunderbolt/GL_Steffan/2016_10/M6'
            self.update_h5_folder_display()
            self.predictions_fname = '/Volumes/G-DRIVE with Thunderbolt/GL_Steffan/2016_10/clf_predictions_m6_201610_no_pks.csv'
            self.update_predictionfile_display()
            self.predictions_df = pd.read_csv(self.predictions_fname)
            self.predictions_df['Index'] = self.predictions_df.index
            print('loading up files for debug')
            for i,row in list(self.predictions_df.iterrows()):
                fpath = os.path.join(self.h5directory,row['Filename'])
                tids = [int(fpath.split(']')[0].split('[')[1])]
                s,e = row['Start'], row['End']
                self.populate_tree_items_list_from_predictions(row, tids)
            self.treeWidget.addTopLevelItems(self.tree_items)

            self.predictions_up = True
            self.library_up = False
            self.file_dir_up = False
        except:
            print('Error, with debug loading')

    def populate_tree_items_list_from_predictions(self, row, tids):
        # todo refactor this name

        self.treeWidget.setColumnCount(5)
        self.treeWidget.setHeaderLabels(['index', 'start', 'end','duration', 'tid', 'fname'])
        filename = row['Filename']
        index =  row['Index']
        start =  row['Start']
        end = row['End']
        duration = row['End']-row['Start']

        fname_entry = [str(filename)]
        details_entry = [str(index),
                         str(start),
                         str(end),
                         str(duration),
                         str(tids[0]), # bad, should make only tid having one explicit
                         str(filename)]
        item = QtGui.QTreeWidgetItem(details_entry)
        item.setFirstColumnSpanned(True)

        self.tree_items.append(item)

        #self.treeWidget.addTopLevelItems(self.tree_items) # now beeing called once all items are there.

    def mouse_click_in_overview(self,evt):
        # signal for this is coming from self.data,
        # evt[0] should be a pyqtgraph.GraphicsScene.mouseEvents.MouseClickEvent

        pos = evt[0].scenePos()
        if self.plot_overview.sceneBoundingRect().contains(pos):
            mousePoint = self.bx_overview.mapSceneToView(pos)
            x = int(mousePoint.x())
            y = int(mousePoint.y())

            xrange, _ = self.get_main_plot_xrange_and_mid()
            try:
                assert xrange > 0
            except:
                print('Your view range is messed up')

            self.plot_1.getViewBox().setXRange(min = x - xrange/2.0,
                                               max = x + xrange/2.0, padding=0)

    def get_main_plot_xrange_and_mid(self):
        xlims  = self.plot_1.getViewBox().viewRange()[0]
        xrange = xlims[1]-xlims[0]
        xmid   = xlims[0]+xrange/2.0
        return xrange, xmid

    def xrange_change(self):
        xrange = self.xrange_spinBox.value()
        if xrange>0:
            _, xmid = self.get_main_plot_xrange_and_mid()
            self.plot_1.getViewBox().setXRange(min = xmid - xrange/2.0,
                                               max = xmid + xrange/2.0, padding=0)
        else:
            pass


    def keyPressEvent(self, eventQKeyEvent):

        key_id = eventQKeyEvent.key()
        key_id_to_numbers = {eval('Qt.Key_'+str(i)):i for i in range(1,10)}
        if key_id in list(key_id_to_numbers.keys()):
            key_val = key_id_to_numbers[key_id]
            self.xrange_spinBox.setValue(key_val)
            #self.xrange_change() # automatically gets called when spin box is hooked up

        x,y = self.plot_1.getViewBox().viewRange()
        if key_id == Qt.Key_Up:
            if self.scroll_flag==True:
                scroll_rate = self.scroll_speed_box.value()
                new_rate = scroll_rate * 2
                self.scroll_speed_box.setValue(new_rate)
                if self.blink ==True: self.reset_timer()
            else:
                self.plot_1.getViewBox().setYRange(min = y[0]*0.9, max = y[1]*0.9, padding = 0)

        if key_id == Qt.Key_Down:
            if self.scroll_flag==True:
                scroll_rate = self.scroll_speed_box.value()
                if scroll_rate > 1:
                    new_rate = int(scroll_rate / 2)
                    self.scroll_speed_box.setValue(new_rate)
                    if self.blink ==True: self.reset_timer()
            else: # just zoom
                self.plot_1.getViewBox().setYRange(min = y[0]*1.1, max = y[1]*1.1,padding = 0)

        if key_id == Qt.Key_Right:
            if self.scroll_flag==True:
                self.scroll_sign = 1
            else:
                scroll_i = (x[1]-x[0])*0.001
                self.plot_1.getViewBox().setXRange(min = x[0]+scroll_i, max = x[1]+scroll_i, padding=0)

        if key_id == Qt.Key_Left:
            if self.scroll_flag==True:
                self.scroll_sign = -1
            else:
                scroll_i = (x[1]-x[0])*0.001
                self.plot_1.getViewBox().setXRange(min = x[0]-scroll_i, max = x[1]-scroll_i, padding=0)

        if key_id == Qt.Key_Backspace or key_id == Qt.Key_Delete:
            current_item = self.treeWidget.currentItem()
            root = self.treeWidget.invisibleRootItem()
            root.removeChild(current_item)

        if key_id == Qt.Key_B:
            if self.scroll_flag==True:
                self.blink *= -1
                if self.blink == True:
                    self.blink_box.setChecked(True)
                else:
                    self.blink_box.setChecked(False)
                self.reset_timer()

        if key_id == Qt.Key_Space:
            self.scroll_sign = 1
            self.scroll_flag *= -1
            self.reset_timer()

    def blink_box_change(self):
        self.reset_timer()
        #print('someone changed the blink box')
    def scroll_speed_change(self):
        self.reset_timer()

    def reset_timer(self):
        scroll_rate = self.scroll_speed_box.value()
        if self.scroll_flag==True:
            self.timer.stop()
            if self.blink_box.isChecked():
                rate = int(2000/scroll_rate)
                #print(rate)
                self.timer.start(rate)
            else:
                self.timer.start(20)
        else:
            self.timer.stop()

    def simple_scroll(self):
        x,y = self.plot_1.getViewBox().viewRange()
        scroll_rate = self.scroll_speed_box.value()
        # the error is
        if self.blink_box.isChecked() != True:
            scroll_i = (x[1]-x[0])*(0.001*scroll_rate)*self.scroll_sign
            new_min =  x[0]+scroll_i
            new_max =  x[1]+scroll_i
            if new_min > 3600: #todo this is hardcoded for the hour long ndfs! We are checking for running over the edge
                self.get_next_tree_item()
            self.plot_1.getViewBox().setXRange(min =new_min, max = new_max, padding=0)
        elif self.blink_box.isChecked():
            scroll_i = (x[1]-x[0])*self.scroll_sign
            new_min =  x[0]+scroll_i
            new_max =  x[1]+scroll_i
            if new_min > 3600: #todo this is hardcoded for the hour long ndfs! We are checking for running over the edge
                self.get_next_tree_item()
            self.plot_1.getViewBox().setXRange(min =new_min, max = new_max, padding=0)

    def load_h5_file(self,fname):

        self.loading_thread = LoadFileThread(fname)
        #self.connect(self.loading_thread, SIGNAL("finished()"), self.done)
        self.connect(self.loading_thread, pyqtSignal("catch_data(PyQt_PyObject)"), self.catch_data)
        self.loading_thread.start()

    def catch_data(self, h5obj):
        self.h5obj = h5obj
        self.plot_traces()


    def done(self):
        QtGui.QMessageBox.information(self, "Done!", "Done loading!")


class LoadFileThread(QThread):

    def __init__(self, filename):
        QThread.__init__(self)
        self.filename = filename

    def __del__(self):
        self.wait()

    def load_file(self, filename):
        self.h5obj = H5File(filename)

    def run(self):
        print('sup, loading: '+self.filename)
        self.load_file(self.filename)
        self.emit(pyqtSignal('catch_data(PyQt_PyObject)'), self.h5obj)

class HDF5Plot(pg.PlotCurveItem):
    """
    Create a subclass of PlotCurveItem for displaying a very large
    data set from an HDF5 file that does not neccesarilly fit in memory.

    The basic approach is to override PlotCurveItem.viewRangeChanged such that it
    reads only the portion of the HDF5 data that is necessary to display the visible
    portion of the data. This is further downsampled to reduce the number of samples
    being displayed.

    A more clever implementation of this class would employ some kind of caching
    to avoid re-reading the entire visible waveform at every update.
    """
    def __init__(self, downsample_limit = 20000,viewbox = None, *args, **kwds):
        " TODO what are the args and kwds for PlotCurveItem class?"
        self.hdf5 = None
        self.time = None
        self.fs = None
        self.vb = viewbox
        self.limit = downsample_limit # maximum number of samples to be plotted, 10000 orginally
        pg.PlotCurveItem.__init__(self, *args, **kwds)
        if pg.CONFIG_OPTIONS['background'] == 'w':
            self.pen = (0,0,0)
        else:
            self.pen = (255,255,255)


    def keyPressEvent(self, event):
        ''' this doesnt work, change key press to correct it.'''
        if event.key() == QtCore.Qt.Key_Escape:
            self.close()
        else:
            pass
            #print(event.key())

    def setHDF5(self, data, time, fs):
        self.hdf5 = data
        self.time = time
        self.fs = fs
        #print ( self.hdf5.shape, self.time.shape)
        self.updateHDF5Plot()

    def viewRangeChanged(self):
        self.updateHDF5Plot()

    def updateHDF5Plot(self):
        if self.hdf5 is None:
            self.setData([])
            return

        #vb = self.getViewBox()
        #if vb is None:
        #    return  # no ViewBox yet

        # Determine what data range must be read from HDF5
        xrange = [i*self.fs for i in self.vb.viewRange()[0]]
        start = max(0,int(xrange[0])-1)
        stop = min(len(self.hdf5), int(xrange[1]+2))

        # Decide by how much we should downsample
        ds = int((stop-start) / self.limit) + 1
        if ds == 1:
            # Small enough to display with no intervention.
            visible_y = self.hdf5[start:stop]
            visible_x = self.time[start:stop]
            scale = 1
        else:
            # Here convert data into a down-sampled array suitable for visualizing.
            # Must do this piecewise to limit memory usage.
            samples = 1 + ((stop-start) // ds)
            visible_y = np.zeros(samples*2, dtype=self.hdf5.dtype)
            visible_x = np.zeros(samples*2, dtype=self.time.dtype)
            sourcePtr = start
            targetPtr = 0

            # read data in chunks of ~1M samples
            chunkSize = (1000000//ds) * ds
            while sourcePtr < stop-1:

                chunk = self.hdf5[sourcePtr:min(stop,sourcePtr+chunkSize)]
                chunk_x = self.time[sourcePtr:min(stop,sourcePtr+chunkSize)]
                sourcePtr += len(chunk)
                #print(chunk.shape, chunk_x.shape)

                # reshape chunk to be integral multiple of ds
                chunk = chunk[:(len(chunk)//ds) * ds].reshape(len(chunk)//ds, ds)
                chunk_x = chunk_x[:(len(chunk_x)//ds) * ds].reshape(len(chunk_x)//ds, ds)

                # compute max and min
                #chunkMax = chunk.max(axis=1)
                #chunkMin = chunk.min(axis=1)

                mx_inds = np.argmax(chunk, axis=1)
                mi_inds = np.argmin(chunk, axis=1)
                row_inds = np.arange(chunk.shape[0])

                chunkMax = chunk[row_inds, mx_inds]
                chunkMin = chunk[row_inds, mi_inds]
                chunkMax_x = chunk_x[row_inds, mx_inds]
                chunkMin_x = chunk_x[row_inds, mi_inds]

                # interleave min and max into plot data to preserve envelope shape
                visible_y[targetPtr:targetPtr+chunk.shape[0]*2:2] = chunkMin
                visible_y[1+targetPtr:1+targetPtr+chunk.shape[0]*2:2] = chunkMax
                visible_x[targetPtr:targetPtr+chunk_x.shape[0]*2:2] = chunkMin_x
                visible_x[1+targetPtr:1+targetPtr+chunk_x.shape[0]*2:2] = chunkMax_x

                targetPtr += chunk.shape[0]*2

            visible_x = visible_x[:targetPtr]
            visible_y = visible_y[:targetPtr]
            #print('**** now downsampling')
            #print(visible_y.shape, visible_x.shape)
            scale = ds * 0.5

        # TODO: setPos, scale, resetTransform methods... scale?


        self.setData(visible_x, visible_y, pen=self.pen) # update the plot
        #self.setPos(start, 0) # shift to match starting index ### Had comment out to stop it breaking... when limit is >0?!
        self.resetTransform()
        #self.scale(scale, 1)  # scale to match downsampling


def main():
    app = QtGui.QApplication(sys.argv)
    form = MainGui()
    form.show()
    app.exec_()

if __name__ == '__main__':
    main()



from distutils.core import setup
setup(
  name = 'pyecog',
  packages = ['pyecog', 'pyecog.ndf','pyecog.light_code', 'pyecog.visualisation'], # this must be the same as the name above
  version = '0.1.38',
  description = 'For classifying single channel ECoG recordings (iEEG)',
  author = 'Jonathan Cornford',
  author_email = 'jonathan.cornford@gmail.com',
  url = 'https://github.com/jcornford/pyecog', # use the URL to the github repo
  download_url = 'https://github.com/jcornford/pyecog/tarball/0.1.38',
  keywords = ['iEEG', 'ECoG'], # arbitrary keywords
  install_requires=['numpy','scipy','scikit-learn','jupyter' ,'pandas', 'matplotlib', 'seaborn','h5py','xlrd', 'bokeh', 'pyqtgraph', 'pomegranate']
)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'check_preds_design_v2.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1046, 728)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.full_splitter = QtWidgets.QSplitter(self.centralwidget)
        self.full_splitter.setOrientation(QtCore.Qt.Vertical)
        self.full_splitter.setObjectName("full_splitter")
        self.top_splitter = QtWidgets.QSplitter(self.full_splitter)
        self.top_splitter.setOrientation(QtCore.Qt.Horizontal)
        self.top_splitter.setObjectName("top_splitter")
        self.treeWidget = QtWidgets.QTreeWidget(self.top_splitter)
        self.treeWidget.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.treeWidget.sizePolicy().hasHeightForWidth())
        self.treeWidget.setSizePolicy(sizePolicy)
        self.treeWidget.setBaseSize(QtCore.QSize(200, 300))
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        self.treeWidget.setFont(font)
        self.treeWidget.setAutoExpandDelay(-1)
        self.treeWidget.setColumnCount(8)
        self.treeWidget.setObjectName("treeWidget")
        self.treeWidget.headerItem().setText(0, "1")
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        self.treeWidget.headerItem().setFont(0, font)
        self.treeWidget.headerItem().setText(1, "2")
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        self.treeWidget.headerItem().setFont(1, font)
        self.treeWidget.headerItem().setText(2, "3")
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        self.treeWidget.headerItem().setFont(2, font)
        self.treeWidget.headerItem().setText(3, "4")
        font = QtGui.QFont()
        font.setFamily("Helvetica Neue")
        self.treeWidget.headerItem().setFont(3, font)
        self.treeWidget.headerItem().setText(4, "5")
        self.treeWidget.headerItem().setText(5, "6")
        self.treeWidget.headerItem().setText(6, "7")
        self.treeWidget.headerItem().setText(7, "8")
        self.treeWidget.header().setDefaultSectionSize(50)
        self.treeWidget.header().setMinimumSectionSize(5)
        self.GraphicsLayoutWidget = GraphicsLayoutWidget(self.top_splitter)
        self.GraphicsLayoutWidget.setEnabled(True)
        self.GraphicsLayoutWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.GraphicsLayoutWidget.setBaseSize(QtCore.QSize(700, 300))
        self.GraphicsLayoutWidget.setObjectName("GraphicsLayoutWidget")
        self.overview_plot = GraphicsLayoutWidget(self.full_splitter)
        self.overview_plot.setMinimumSize(QtCore.QSize(0, 0))
        self.overview_plot.setBaseSize(QtCore.QSize(912, 200))
        self.overview_plot.setObjectName("overview_plot")
        self.bottom_splitter = QtWidgets.QSplitter(self.full_splitter)
        self.bottom_splitter.setOrientation(QtCore.Qt.Horizontal)
        self.bottom_splitter.setObjectName("bottom_splitter")
        self.layoutWidget = QtWidgets.QWidget(self.bottom_splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.layoutWidget)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_2 = QtWidgets.QLabel(self.layoutWidget)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 1, 2, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.layoutWidget)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 1, 0, 1, 1)
        self.h5_folder_display = QtWidgets.QLineEdit(self.layoutWidget)
        self.h5_folder_display.setMinimumSize(QtCore.QSize(0, 0))
        self.h5_folder_display.setMaximumSize(QtCore.QSize(16777215, 100))
        self.h5_folder_display.setDragEnabled(True)
        self.h5_folder_display.setReadOnly(True)
        self.h5_folder_display.setCursorMoveStyle(QtCore.Qt.VisualMoveStyle)
        self.h5_folder_display.setObjectName("h5_folder_display")
        self.gridLayout_3.addWidget(self.h5_folder_display, 1, 1, 1, 1)
        self.scroll_speed_box = QtWidgets.QSpinBox(self.layoutWidget)
        self.scroll_speed_box.setMaximumSize(QtCore.QSize(80, 16777215))
        self.scroll_speed_box.setMinimum(1)
        self.scroll_speed_box.setObjectName("scroll_speed_box")
        self.gridLayout_3.addWidget(self.scroll_speed_box, 0, 3, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.layoutWidget)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 0, 0, 1, 1)
        self.predictions_file_display = QtWidgets.QLineEdit(self.layoutWidget)
        self.predictions_file_display.setMinimumSize(QtCore.QSize(0, 0))
        self.predictions_file_display.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.predictions_file_display.setObjectName("predictions_file_display")
        self.gridLayout_3.addWidget(self.predictions_file_display, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.layoutWidget)
        self.label.setObjectName("label")
        self.gridLayout_3.addWidget(self.label, 0, 2, 1, 1)
        self.blink_box = QtWidgets.QCheckBox(self.layoutWidget)
        self.blink_box.setObjectName("blink_box")
        self.gridLayout_3.addWidget(self.blink_box, 1, 3, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.layoutWidget)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 2, 2, 1, 1)
        self.xrange_spinBox = QtWidgets.QSpinBox(self.layoutWidget)
        self.xrange_spinBox.setMaximumSize(QtCore.QSize(80, 16777215))
        self.xrange_spinBox.setMinimum(1)
        self.xrange_spinBox.setProperty("value", 5)
        self.xrange_spinBox.setObjectName("xrange_spinBox")
        self.gridLayout_3.addWidget(self.xrange_spinBox, 2, 3, 1, 1)
        self.tid_spinBox = QtWidgets.QSpinBox(self.layoutWidget)
        self.tid_spinBox.setMaximumSize(QtCore.QSize(80, 16777215))
        self.tid_spinBox.setObjectName("tid_spinBox")
        self.gridLayout_3.addWidget(self.tid_spinBox, 2, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.layoutWidget)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 2, 0, 1, 1)
        self.textBrowser = QtWidgets.QTextBrowser(self.bottom_splitter)
        self.textBrowser.setMaximumSize(QtCore.QSize(400, 16777215))
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout_2.addWidget(self.full_splitter)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menuBar = QtWidgets.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 1046, 22))
        self.menuBar.setObjectName("menuBar")
        self.menuFile = QtWidgets.QMenu(self.menuBar)
        self.menuFile.setObjectName("menuFile")
        self.menuAnalyse = QtWidgets.QMenu(self.menuBar)
        self.menuAnalyse.setObjectName("menuAnalyse")
        MainWindow.setMenuBar(self.menuBar)
        self.actionSave_annotations = QtWidgets.QAction(MainWindow)
        self.actionSave_annotations.setObjectName("actionSave_annotations")
        self.actionLoad_Library = QtWidgets.QAction(MainWindow)
        self.actionLoad_Library.setObjectName("actionLoad_Library")
        self.actionLoad_Predictions = QtWidgets.QAction(MainWindow)
        self.actionLoad_Predictions.setObjectName("actionLoad_Predictions")
        self.actionLoad_h5_folder = QtWidgets.QAction(MainWindow)
        self.actionLoad_h5_folder.setObjectName("actionLoad_h5_folder")
        self.actionConvert_dir_to_h5 = QtWidgets.QAction(MainWindow)
        self.actionConvert_dir_to_h5.setObjectName("actionConvert_dir_to_h5")
        self.actionConvert_ndf_to_h5 = QtWidgets.QAction(MainWindow)
        self.actionConvert_ndf_to_h5.setObjectName("actionConvert_ndf_to_h5")
        self.actionLibrary_logistics = QtWidgets.QAction(MainWindow)
        self.actionLibrary_logistics.setObjectName("actionLibrary_logistics")
        self.actionAdd_to_library = QtWidgets.QAction(MainWindow)
        self.actionAdd_to_library.setObjectName("actionAdd_to_library")
        self.actionAdd_labels_to_library = QtWidgets.QAction(MainWindow)
        self.actionAdd_labels_to_library.setObjectName("actionAdd_labels_to_library")
        self.actionAdd_features_to_library = QtWidgets.QAction(MainWindow)
        self.actionAdd_features_to_library.setObjectName("actionAdd_features_to_library")
        self.actionClassifier_components = QtWidgets.QAction(MainWindow)
        self.actionClassifier_components.setObjectName("actionClassifier_components")
        self.actionRun_classifer_on_h5_dir = QtWidgets.QAction(MainWindow)
        self.actionRun_classifer_on_h5_dir.setObjectName("actionRun_classifer_on_h5_dir")
        self.actionRun_classifer_on_ndf_dir = QtWidgets.QAction(MainWindow)
        self.actionRun_classifer_on_ndf_dir.setObjectName("actionRun_classifer_on_ndf_dir")
        self.actionSet_default_folder = QtWidgets.QAction(MainWindow)
        self.actionSet_default_folder.setObjectName("actionSet_default_folder")
        self.actionAdd_features_to_h5_folder = QtWidgets.QAction(MainWindow)
        self.actionAdd_features_to_h5_folder.setObjectName("actionAdd_features_to_h5_folder")
        self.menuFile.addAction(self.actionLoad_Library)
        self.menuFile.addAction(self.actionLoad_Predictions)
        self.menuFile.addAction(self.actionLoad_h5_folder)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSave_annotations)
        self.menuFile.addAction(self.actionSet_default_folder)
        self.menuAnalyse.addAction(self.actionConvert_ndf_to_h5)
        self.menuAnalyse.addSeparator()
        self.menuAnalyse.addAction(self.actionAdd_features_to_h5_folder)
        self.menuAnalyse.addSeparator()
        self.menuAnalyse.addAction(self.actionLibrary_logistics)
        self.menuAnalyse.addSeparator()
        self.menuAnalyse.addAction(self.actionClassifier_components)
        self.menuBar.addAction(self.menuFile.menuAction())
        self.menuBar.addAction(self.menuAnalyse.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label_2.setText(_translate("MainWindow", "Scroll type"))
        self.label_4.setText(_translate("MainWindow", "h5 Folder:"))
        self.label_3.setText(_translate("MainWindow", "Prediction File:"))
        self.label.setText(_translate("MainWindow", "Scroll speed"))
        self.blink_box.setText(_translate("MainWindow", "Blinking"))
        self.label_5.setText(_translate("MainWindow", "Xrange (sec)"))
        self.label_6.setText(_translate("MainWindow", "Tid for h5 folder "))
        self.textBrowser.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.SF NS Text\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt; font-weight:600;\">Mouse Functionality</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\">Scroll when hovering over either axis in the main plot to zoom that axis only. Scrolling when in the middle will zoom both axes at the same time - you probably don\'t want that.</span></p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt; font-weight:600;\">Keyboard Shortcuts</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\">Number key:   Set plot time interval </span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\">B key:              Toggle blink vs scroll</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\">SPACE:           Start scrolling (or blinking)</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\">Up:        Zoom in   / speed up              </span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\">Down:        Zoom out / slow down              </span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\">Right arrow:    Step right / scroll forwards      </span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\">Left arrow:        Step left   / scroll backwards</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\"><br /></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\"><br /></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\"><br /></p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'.Helvetica Neue DeskInterface\'; font-size:14pt;\"><br /></p></body></html>"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuAnalyse.setTitle(_translate("MainWindow", "Analyse"))
        self.actionSave_annotations.setText(_translate("MainWindow", "Save annotations"))
        self.actionLoad_Library.setText(_translate("MainWindow", "Load Library"))
        self.actionLoad_Predictions.setText(_translate("MainWindow", "Load Predictions"))
        self.actionLoad_h5_folder.setText(_translate("MainWindow", "Load h5 folder"))
        self.actionConvert_dir_to_h5.setText(_translate("MainWindow", "Convert dir to h5"))
        self.actionConvert_ndf_to_h5.setText(_translate("MainWindow", "Convert ndf folder to h5"))
        self.actionLibrary_logistics.setText(_translate("MainWindow", "Library logistics"))
        self.actionAdd_to_library.setText(_translate("MainWindow", "Add to library"))
        self.actionAdd_labels_to_library.setText(_translate("MainWindow", "Add labels to library"))
        self.actionAdd_features_to_library.setText(_translate("MainWindow", "Add features to library"))
        self.actionClassifier_components.setText(_translate("MainWindow", "Classifier components"))
        self.actionRun_classifer_on_h5_dir.setText(_translate("MainWindow", "Run classifer on h5 dir"))
        self.actionRun_classifer_on_ndf_dir.setText(_translate("MainWindow", "Run classifer on ndf dir"))
        self.actionSet_default_folder.setText(_translate("MainWindow", "Set default folder"))
        self.actionAdd_features_to_h5_folder.setText(_translate("MainWindow", "Add features to h5 folder"))

from pyqtgraph import GraphicsLayoutWidget



"""
This file is now out of date. Use network_classifier_v2.

This was last run to save the training data into a hdf5 file.

"""

import pickle

import h5py
import numpy as np

import utils
from pyecog.light_code.extractor import FeatureExtractor
from pyecog.light_code.classifier import NetworkClassifer


################# Training Data ###################
reload_training = False
if reload_training:
    training_traces = utils.raw_training_load()
    training_traces_norm = utils.normalise(training_traces)
    training_data = FeatureExtractor(training_traces_norm)
    #f = open('../full_raw_training','wb')
    #pickle.dump(training_traces,f)

elif not reload_training:
    print 'skipping raw training load'
    training_traces = pickle.load(open('../full_raw_training','rb'))
    training_traces_norm = utils.normalise(training_traces)
    training_data = FeatureExtractor(training_traces_norm)
    np.savetxt('training_traces.csv',training_traces_norm,delimiter=',')

################# Training Labels and mixed event exclusion ###################
cleanup = np.loadtxt('../Training_cleanup.csv',delimiter=',')
training_labels = np.array([int(x[1]) for x in cleanup])
print training_labels.shape
training_indexes = []
for i in range(training_labels.shape[0]):
        if training_labels[i] != 0:
            training_indexes.append(i)


################## Training data added 2016/02/15 after first try outs ################
lacie_training = '/Volumes/LACIE SHARE/VM_data/All_Data_Jan_2016/labelling_for_training/'
training_traces2 = pickle.load(open(lacie_training + 'new_training_data_2016_02_09','rb'))
training_traces2_norm = utils.normalise(training_traces2)
training_data2 = FeatureExtractor(training_traces2_norm).feature_array
training_labels2 = pickle.load(open(lacie_training + 'new_training_labels_2016_02_09','rb'))

# now merge:

updated_training_data_traces = np.vstack((training_traces[training_indexes,:],training_traces2))
updated_training_data = np.vstack((training_data.feature_array[training_indexes,:], training_data2))
updated_training_labels = np.hstack((training_labels[training_indexes],training_labels2))
print updated_training_data.shape, 'is updated shape, and labels...',
print updated_training_labels.shape
print updated_training_data_traces.shape, 'is updated traces shape!'



################## Test Data ####################
reload_validation = False
if reload_validation:
    validation_traces = utils.raw_validation_load()
    validation_traces_norm = utils.normalise(validation_traces)
    validation_data = FeatureExtractor(validation_traces_norm)
    #f = open('../raw_validation','wb')
    #pickle.dump(validation_traces,f)

elif not reload_training:
    print 'skipping raw validation load'
    validation_traces = pickle.load(open('../raw_validation','rb'))
    validation_traces_norm = utils.normalise(validation_traces)
    validation_data = FeatureExtractor(validation_traces_norm)
    np.savetxt('test_traces.csv',validation_traces_norm,delimiter=',')

################# Validation cleanup ###################
cleanup = np.loadtxt('../validation_cleanUp.csv',delimiter=',')
validation_labels = np.array([int(x[1]) for x in cleanup])

validation_indexes = []
for i in range(validation_labels.shape[0]):
        if validation_labels[i] != 0:
            validation_indexes.append(i)

#np.savetxt('all_traces.csv',np.vstack((validation_traces_norm,training_traces_norm)),delimiter=',')
#np.savetxt('all_cleanup.csv',np.hstack((validation_labels)), delimiter = ',')

# comment out on 2016/02/15 when using the extra training data!
#classifier = NetworkClassifer(training_data.feature_array[training_indexes,:],training_labels[training_indexes],
#                             validation_data.feature_array[validation_indexes],validation_labels[validation_indexes])

###### Save the training and test here into hdf5 / databases #######
hdf5_data = {}
#file_name = '/Volumes/LACIE SHARE/VM_data/classifier_hdf5data/classifier_test_train_20160223.hdf5'

print file_name

with h5py.File(file_name, 'w') as f:
    f.name

    training = f.create_group('training')
    test = f.create_group('test')

    training.create_dataset('data', data = updated_training_data_traces)
    test.create_dataset('data', data = validation_traces[validation_indexes])

    training.create_dataset('labels', data = updated_training_labels)
    test.create_dataset('labels', data = validation_labels[validation_indexes])

    training.create_dataset('features', data = updated_training_data)
    test.create_dataset('features',data = validation_data.feature_array[validation_indexes])

    print training.keys()
    print test.keys()

    #plt.plot(training['data'][49,:])

#exit()

classifier = NetworkClassifer(updated_training_data,updated_training_labels,
                              validation_data.feature_array[validation_indexes],validation_labels[validation_indexes])
classifier.run()

classifier.pca(n_components = 3)
classifier.lda(n_components = 3, pca_reg = False, reg_dimensions = 9)
classifier.lda_run()
classifier.pca_run()
#classifier.randomforest_info(max_trees=2000, step = 50)
f = open('../pickled_classifier','wb')
pickle.dump(classifier,f)

''' ######## for pdfs of the validation ############ '''
#f = open('../validation_label_traces_tuple','wb')
#pickle.dump((validation_labels ,datasettest_norm[ok_indexes,:]),f)

'''######## for pdfs of the original features ############'''
#training_traces = np.vstack((data0616,dataset301_norm))
#training_traces = training_data.feature_array[ok_indexes,:]
#f = open('../training_label_traces_tuple','wb')
#pickle.dump((training_indexes,training_traces),f)

'''
testname_list = [testdataobj.filename_list[i] for i in range(testdataobj.filename_list.shape[0])]
len(testname_list)

info = []
for i, name in enumerate(testname_list):
    #print i,name[-34:]," State:" +str(pred_labels_new[i]), str(np.max(pred_labels_new_pr[i,:])*100)+'%'
    info.append([i,name[-40:]," State:" +str(pred_labels_new[i]), str(np.max(pred_labels_new_pr[i,:])*100)+'%'])
info = np.array(info)
print info.shape

######### Feature selection ###########
feature_labels = ['0min','1max','2mean','3skew','4std','5kurtosis','6sum of absolute difference','7baseline_n',
                               '8baseline_diff','9baseline_diff_skew','10n_pks','11n_vals','12av_pk','13av_val','14av pk val range',
                               '151 hz','165 hz','1710 hz','1815 hz','1920 hz','20_30 hz','21_60 hz','22_90 hz']
fis = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22]
fis = [0,1,2,3,4,5,6,7,8,9,12,14,15,16,17,18,19]

'''


## adapted from http://stackoverflow.com/questions/16824718/python-matplotlib-pyside-fast-timetrace-scrolling

from pyqtgraph.Qt import QtGui, QtCore
import numpy as np
import pyqtgraph as pg

N_SAMPLES = 100000

def test_plot():
    time = np.arange(N_SAMPLES)*1e-3
    sample = np.random.randn(N_SAMPLES)

    plt = pg.PlotWidget(title="Use the slider to scroll and the spin-box to set the width")
    plt.addLegend()
    plt.plot(time, sample, name="Gaussian noise")
    q = ScrollingToolQT(plt)
    return q   # WARNING: it's important to return this object otherwise
            # python will delete the reference and the GUI will not respond!


class ScrollingToolQT(object):
    def __init__(self, fig):
        # Setup data range variables for scrolling
        self.fig = fig
        self.xmin, self.xmax = fig.plotItem.vb.childrenBounds()[0]
        self.step = 1 # axis units

        self.scale = 1e3 # conversion betweeen scrolling units and axis units

        # Retrive the QMainWindow used by current figure and add a toolbar
        # to host the new widgets
        self.win = QtGui.QMainWindow()
        self.win.show()
        self.win.resize(800,600)
        self.win.setCentralWidget(fig)
        self.toolbar = QtGui.QToolBar()
        self.win.addToolBar(QtCore.Qt.BottomToolBarArea, self.toolbar)

        # Create the slider and spinbox for x-axis scrolling in toolbar
        self.set_slider(self.toolbar)
        self.set_spinbox(self.toolbar)

        # Set the initial xlimits coherently with values in slider and spinbox
        self.set_xlim = self.fig.setXRange
        self.set_xlim(0, self.step)

    def set_slider(self, parent):
        # Slider only support integer ranges so use ms as base unit
        smin, smax = self.xmin*self.scale, self.xmax*self.scale

        self.slider = QtGui.QSlider(QtCore.Qt.Horizontal, parent=parent)
        self.slider.setTickPosition(QtGui.QSlider.TicksAbove)
        self.slider.setTickInterval((smax-smin)/10.)
        self.slider.setMinimum(smin)
        self.slider.setMaximum(smax-self.step*self.scale)
        self.slider.setSingleStep(self.step*self.scale/5.)
        self.slider.setPageStep(self.step*self.scale)
        self.slider.setValue(0)  # set the initial position
        self.slider.valueChanged.connect(self.xpos_changed)
        parent.addWidget(self.slider)

    def set_spinbox(self, parent):
        self.spinb = QtGui.QDoubleSpinBox(parent=parent)
        self.spinb.setDecimals(3)
        self.spinb.setRange(0.001, 3600.)
        self.spinb.setSuffix(" s")
        self.spinb.setValue(self.step)   # set the initial width
        self.spinb.valueChanged.connect(self.xwidth_changed)
        parent.addWidget(self.spinb)

    def xpos_changed(self, pos):
        #pprint("Position (in scroll units) %f\n" %pos)
        #        self.pos = pos/self.scale
        pos /= self.scale
        self.set_xlim(pos, pos + self.step, padding=0)

    def xwidth_changed(self, xwidth):
        #pprint("Width (axis units) %f\n" % step)
        if xwidth <= 0: return
        self.step = xwidth
        self.slider.setSingleStep(self.step*self.scale/5.)
        self.slider.setPageStep(self.step*self.scale)
        old_xlim = self.fig.plotItem.vb.viewRange()[0]
        self.xpos_changed(old_xlim[0] * self.scale)

if __name__ == "__main__":
    app = pg.mkQApp()
    q = test_plot()
    app.exec_()


import time
import sys
import logging
import warnings
import numpy as np
import scipy.stats as stats

class StdDevStandardiser():
    def __init__(self,data, std_sigfigs = 2):
        '''
        Calculates mode std dev and divides by it.

        Args:
            data:
            stdtw: time period over which to calculate std deviation
            std_sigfigs: n signfigs to round to

        '''
        logging.debug('Standardising dataset to mode std dev')
        #std_vector = self.round_to_sigfigs(np.std(data, axis = 1), sigfigs=std_sigfigs)
        std_vector = np.round(np.std(data, axis = 1), 0)
        #logging.debug(str(std_vector))
        std_vector = std_vector[std_vector != 0]
        if std_vector.shape[0] > 0:
            self.mode_std = stats.mode(std_vector)[0] # can be zero if there is big signal loss
            self.scaled = np.divide(data, self.mode_std)
            logging.debug(str(self.mode_std)+' is mode std of trace split into ')
        elif std_vector.shape[0] == 0:
            self.scaled = None
            logging.error(' File std is all 0, changed data to be None')

    @staticmethod
    def round_to_sigfigs(x, sigfigs):
        """
        N.B Stolen from stack overflow:
        http://stackoverflow.com/questions/18915378/rounding-to-significant-figures-in-numpy

        Rounds the value(s) in x to the number of significant figures in sigfigs.
        Restrictions:
        sigfigs must be an integer type and store a positive value.
        x must be a real value or an array like object containing only real values.
        """
        #The following constant was computed in maxima 5.35.1 using 64 bigfloat digits of precision
        __logBase10of2 = 3.010299956639811952137388947244930267681898814621085413104274611e-1
        if not ( type(sigfigs) is int or np.issubdtype(sigfigs, np.integer)):
            raise TypeError( "RoundToSigFigs: sigfigs must be an integer." )
        if not np.all(np.isreal( x )):
            raise TypeError( "RoundToSigFigs: all x must be real." )
        if sigfigs <= 0:
            raise ValueError( "RoundtoSigFigs: sigfigs must be positive." )
        mantissas, binaryExponents = np.frexp(x)
        decimalExponents = __logBase10of2 * binaryExponents
        intParts = np.floor(decimalExponents)
        mantissas *= 10.0**(decimalExponents - intParts)
        return np.around(mantissas, decimals=sigfigs - 1 ) * 10.0**intParts

class FeatureExtractor():
    '''
    TODO:
     - Window on the baseline should be fs dependent
     - store the conversion factor for the scaling to std/ implement this all here, not in the
     - pandas df?
     - peakdet method is very slow, replace with  https://gist.github.com/sixtenbe/1178136? does it add much?
     - use of self and basically staticmethods is all jumbled up
     - flat data and chunked array is a little jumbled too
     - skew and mean diff of baseline is not actually working
    '''

    def __init__(self, data, fs, extract = True, run_peakdet = True):
        '''
        Inputs:
            - Data: should already be chunked into desired windows. Assumes this is on hour's worth.
            - fs: sampling frequency
            - extract: a flag to extract all the features - default true

        N.B Scaling to unit std should be done here!
        '''
        # todo do not assume an hour?

        self.start = time.clock()
        standardiser = StdDevStandardiser(data,std_sigfigs=2)
        self.dataset = standardiser.scaled
        self.mode_std = standardiser.mode_std

        self.chunk_len = 3600/self.dataset.shape[0] # get this as an arg?
        self.flat_data = np.ravel(self.dataset, order = 'C')
        self.fs = fs

        logging.debug('Fs passed to Feature extractor was: '+str(fs)+' hz')
        # add in 120-160 if we have the fs for it:
        if self.fs < 512:
            self.powerbands = [(1,4),(4,8),(8,12),(12,30),(30,50),(50,70),(70,120)]
            self.powerband_titles = [str(i)+'-'+str(ii)+' Hz' for i,ii in self.powerbands]

        elif self.fs >= 512 :
            self.powerbands = [(1,4),(4,8),(8,12),(12,30),(30,50),(50,70),(70,120), (120,160)]
            self.powerband_titles = [str(i)+'-'+str(ii)+' Hz' for i,ii in self.powerbands]

        if extract:
            self.get_basic_stats()
            self.baseline(threshold = 1.5, window_size = 80) # window size needs to be frequency dependent

            self.bandpow_arr = self.calculate_powerbands(self.flat_data,
                                                         chunk_len=self.chunk_len,
                                                         fs = self.fs,
                                                         bands = self.powerbands)
            # this split is clunky, change this
            if run_peakdet:
                self.peaks_and_valleys(delta_val = 3.0)
                self.feature_array = np.hstack([self.basic_stats,
                                            self.baseline_features,
                                            self.pkval_stats,
                                            self.bandpow_arr])

                self.col_labels = np.hstack([self.basic_stats_col_labels,
                                             self.baseline_features_labels,
                                             self.pkval_labels,
                                             self.powerband_titles])
            else:
                self.feature_array = np.hstack([self.basic_stats,
                                            self.baseline_features,
                                            self.bandpow_arr])

                self.col_labels = np.hstack([self.basic_stats_col_labels,
                                             self.baseline_features_labels,
                                             self.powerband_titles])

            timetaken = np.round((time.clock()-self.start), 2)
            logging.debug("Stacking up..."+str(self.feature_array.shape))
            logging.debug('Took '+ str(timetaken) + ' seconds to extract '+ str(self.dataset.shape[0])+ ' feature vectors')

    def get_basic_stats(self):
        logging.debug("Extracting basic stats...")
        self.basic_stats_col_labels = ('min', 'max', 'mean', 'std-dev', 'kurtosis', 'skew', 'sum(abs(difference))')
        self.basic_stats = np.zeros((self.dataset.shape[0],7))
        self.basic_stats[:,0] = np.amin(self.dataset, axis = 1)
        self.basic_stats[:,1] = np.amax(self.dataset, axis = 1)
        self.basic_stats[:,2] = np.mean(self.dataset, axis = 1)
        self.basic_stats[:,3] = np.std(self.dataset, axis = 1)
        self.basic_stats[:,4] = stats.kurtosis(self.dataset,axis = 1)
        self.basic_stats[:,5] = stats.skew(self.dataset, axis = 1)
        self.basic_stats[:,6] = np.sum(np.absolute(np.diff(self.dataset, axis = 1)),axis = 1)

        # get rid of pesky NaNs, from interpolating over corruptions etc, in more controlled way than just imputing later
        self.basic_stats[:,4][np.isnan(self.basic_stats[:,4])] = -3.0
        self.basic_stats[:,5][np.isnan(self.basic_stats[:,5])] = 0.0
        self.basic_stats[:,4][np.isinf(self.basic_stats[:,4])] = -3.0
        self.basic_stats[:,5][np.isinf(self.basic_stats[:,5])] = 0.0


    @staticmethod
    def calculate_powerbands(flat_data, chunk_len, fs, bands):
        '''
        Inputs:
            - flat_data: np.array, of len(arr.shape) = 1
            - chunk_len: length in seconds of chunk over which to calculate bp
            - fs       : sampling frequency in Hz
            - bands    : list of tuples, containing bands of interest.
            e.g. [(1,4),(4,8),(8,12),(12,30),(30,50),(50,70),(70,120)]

        Returns:
            - numpy array, columns correspond to powerbands.
            Maybe could switch out for pandas dataframe in future?

        Notes:
            - Band power units are "y unit"^2
            - using rfft for slightly improved efficiency - a scaling factor for power
            computation needs to be included
            - using a hanning window, so reflecting half of the first and last chunk in
            order to centre the window over the time windows of interest (and to not throw
            anything away)

        '''

        # first reflect the first and last half chunk length so we don't lose any time
        pad_dp = int((chunk_len/2) * fs)
        padded_data = np.pad(flat_data, pad_width=pad_dp, mode = 'reflect')

        # reshape data into array, stich together
        data_arr = np.reshape(padded_data,
                              newshape=(int(len(padded_data)/fs/chunk_len), int(chunk_len*fs)), # changed in order to compensate for files which are not 3600seconds long
                              order= 'C')
        data_arr_stiched = np.concatenate([data_arr[:-1], data_arr[1:]], axis = 1)

        # window the data with hanning window
        hanning_window = np.hanning(data_arr_stiched.shape[1])
        windowed_data  = np.multiply(data_arr_stiched, hanning_window)

        # run fft and get psd
        bin_frequencies = np.fft.rfftfreq(int(chunk_len*fs)*2, 1/fs) # *2 as n.points is doubled due to stiching
        A = np.abs(np.fft.rfft(windowed_data, axis = 1))
        psd = 2*chunk_len*(2/.375)*(A/(fs*chunk_len*2))**2  # psd units are "y unit"^2/Hz
        # A/(fs*chunk_len*2) is the normalised fft transform of windowed_data
        # - rfft only returns the positive frequency amplitudes.
        # To compute the power we have to sum the squares of both positive and negative complex frequency amplitudes
        # .375 is the power of the Hanning window - the factor 2/.375 corrects for these two points.
        # 2*chunk_len - is a factor such that the result comes as a density with units "y unit"^2/Hz and not just "y unit"^2
        # this is just to make the psd usable for other purposes - in the next section bin_width*2*chunk_len equals 1.

        # now grab power bands from psd
        bin_width = np.diff(bin_frequencies[:3])[1] # this way is really not needed?
        pb_array = np.zeros(shape = (psd.shape[0],len(bands)))
        for i,band in enumerate(bands):
            lower_freq, upper_freq = band # unpack the band tuple
            band_indexes = np.where(np.logical_and(bin_frequencies > lower_freq, bin_frequencies<=upper_freq))[0]
            bp = np.sum(psd[:,band_indexes], axis = 1)*bin_width
            pb_array[:,i] = bp

        return pb_array

    @staticmethod
    def rolling_window(array, window):
        """
        Remember that the rolling window actually starts at the window length in.
        """
        shape = array.shape[:-1] + (array.shape[-1] - window + 1, window)
        strides = array.strides + (array.strides[-1],)
        return np.lib.stride_tricks.as_strided(array, shape=shape, strides=strides)

    def baseline(self, threshold = 1.5, window_size = 80):
        '''
        #TODO: Window size should be fs dependent?
        - - `threshold has been chosen assuming the data has been scaled to mode std dev

        - used to have mean baseline diff and also baseline diff skew for vincents model
          kept, diff (with a fudge for when few datapoints, but not skew.) Though from looking,
          skew seems very predictive of pre seizure?

        - keep in
        '''
        data = self.dataset

        flat_data = np.ravel(data, order = 'C')
        array_std = np.std(self.rolling_window(flat_data, window=window_size),-1)
        array_window_missing = np.zeros(window_size-1) # goes on at the front
        rolling_std_array = np.hstack((array_window_missing,array_std))

        below_threshold_masked = np.ma.masked_where(rolling_std_array > threshold, flat_data)
        above_threshold_masked = np.ma.masked_where(rolling_std_array < threshold, flat_data)
        # masked_where puts a mask where the condition is met, so sign is other way to what you would expect
        # when using indexing, i.e.: above_threshold = flat_data[np.where(rolling_std_array > threshold)[0]]

        below_std_arr = np.reshape(below_threshold_masked, newshape = data.shape, order = 'C')
        above_std_arr = np.reshape(above_threshold_masked, newshape = data.shape, order = 'C')

        baseline_length = below_std_arr.count(axis = 1) # .count counts numb of non masked elements
        #print(baseline_length[:10])

        # build indexes array:
        indexes_row = np.arange(data.shape[1])
        indexes_arr = np.empty(data.shape)
        for i in range(data.shape[0]): indexes_arr[i, :] = indexes_row

        bl_indexes = np.ma.masked_array(indexes_arr, mask = below_std_arr.mask)

        # have to use lists and compressed, or diff ignores the jump
        # this is all very suboptimal feature engineering - MARCO?
        diff_arr_list = [np.diff(bl_indexes[i,:].compressed()) for i in range(data.shape[0])]
        baseline_index_diff_skew = np.array([stats.stats.skew(diff_arr_list[i]) for i in range(data.shape[0])])
        with warnings.catch_warnings():# suppress the mean of empty slice warning
            warnings.simplefilter("ignore", category=RuntimeWarning)
            baseline_mean_diff       = np.array([np.mean(diff_arr_list[i]) for i in range(data.shape[0])])

        # hereshould make diff array once - it is masked itself
        #baseline_index_diff_skew = stats.mstats.skew(np.diff(bl_indexes, axis = 1), axis = 1)
        #baseline_mean_diff       = np.mean(np.diff(bl_indexes, axis = 1), axis = 1)

        # below is to handle when you have very few baseline points..
        #  Not actually that satisfactory.
        less_bl_datapoints = np.where(baseline_length<100)[0]
        baseline_mean_diff[less_bl_datapoints] = self.fs/(baseline_length[less_bl_datapoints]+2) # a hack to overcome if no baselines...
        baseline_index_diff_skew[less_bl_datapoints] = 0.0

        #same just in case mask slips through somehow
        #if baseline_mean_diff.mask.any():
        #    baseline_mean_diff[baseline_mean_diff.mask] = self.fs/(baseline_length[less_bl_datapoints]+2)
        #if baseline_index_diff_skew.mask.any():
        #    baseline_index_diff_skew[baseline_index_diff_skew.mask]  = 0.0

        self.baseline_features = np.vstack([baseline_length, baseline_mean_diff,baseline_index_diff_skew]).T
        self.baseline_features_labels = ['bl_len','bl_mean_diff', 'bl_skew']
        #print(self.baseline_features)

    def peaks_and_valleys(self, delta_val = 3.0):
        '''underlying peak det function is slow...
        the range and values dont seem to be that great...
        number a bit more interesting? nval and npk is often same but not alowas one to one

        Should really be using:
        https://gist.github.com/sixtenbe/1178136
        '''

        logging.debug("Extracting peak/valley features...")
        pk_nlist = []
        val_nlist = []

        for i in range(self.dataset.shape[0]):
            pk, val = self.peakdet(self.dataset[i,:], delta=delta_val)
            pk_nlist.append(pk)
            val_nlist.append(val)

        # always 2 columns
        n_pks = []
        n_vals = []
        av_pk = []
        av_val = []
        av_range = []
        for i in range(len(pk_nlist)):
            n_pks.append(pk_nlist[i].shape[0])
            n_vals.append(val_nlist[i].shape[0])

            if pk_nlist[i].shape[0]:
                av_pk.append(np.mean(pk_nlist[i][:,1]))
            else:
                av_pk.append(np.NaN)

            if val_nlist[i].shape[0]:
                av_val.append(np.mean(val_nlist[i][:,1]))
            else:
                av_val.append(np.NaN)

            if val_nlist[i].shape[0] and pk_nlist[i].shape[0]:
                av_range.append(
                    abs(np.mean(pk_nlist[i][:,1])+np.mean(val_nlist[i][:,1]))
                    )
            else:
                av_range.append(np.NaN)

        n_pks = np.array(n_pks)
        n_vals = np.array(n_vals)
        av_pk = np.array(av_pk)
        av_val = np.array(av_val)
        av_range = np.array(av_range)
        self.pkval_stats = np.vstack([n_pks,n_vals,av_pk,av_val,av_range]).T
        col_mins = np.nanmin(self.pkval_stats, axis = 0)
        col_mins[3] = np.nanmax(self.pkval_stats[:,3], axis =0) # hacky
        for row, col in np.argwhere(np.isnan(self.pkval_stats)):
            self.pkval_stats[row,col] = col_mins[col]

        self.pkval_labels = ['n_pks','n_vals','av_pk','av_val','av_pkval_range']

    @staticmethod
    def peakdet(v, delta, x=None):
        """
        function [maxtab, mintab]=peakdet(v, delta, x)
                [MAXTAB, MINTAB] = PEAKDET(V, DELTA) finds the local
                maxima and minima ("peaks") in the vector V.
                MAXTAB and MINTAB consists of two columns. Column 1
                contains indices in V, and column 2 the found values.

                With [MAXTAB, MINTAB] = PEAKDET(V, DELTA, X) the indices
                in MAXTAB and MINTAB are replaced with the corresponding
               X-values.

                A point is considered a maximum peak if it has the maximal
                value, and was preceded (to the left) by a value lower by
                DELTA.
        """
        maxtab = []
        mintab = []
        if x is None:
            x = np.arange(len(v))
        v = np.asarray(v)
        if len(v) != len(x):
            sys.exit('Input vectors v and x must have same length')
        if not np.isscalar(delta):
            sys.exit('Input argument delta must be a scalar')
        if delta <= 0:
            sys.exit('Input argument delta must be positive')
        mn, mx = np.Inf, -np.Inf
        mnpos, mxpos = np.NaN, np.NaN
        lookformax = True
        for i in np.arange(len(v)):
            this = v[i]
            if this > mx:
                mx = this
                mxpos = x[i]
            if this < mn:
                mn = this
                mnpos = x[i]
            if lookformax:
                if this < mx-delta:
                    maxtab.append((mxpos, mx))
                    mn = this
                    mnpos = x[i]
                    lookformax = False
            else:
                if this > mn+delta:
                    mintab.append((mnpos, mn))
                    mx = this
                    mxpos = x[i]
                    lookformax = True
        return np.array(maxtab), np.array(mintab)
